package httpUtils

import (
	"net/http"
	"time"
)

type Request struct {
	method   string
	url      string
	params   map[string]interface{}
	headers  map[string]string
	body     any
	bodyJSON any
	timeout  time.Duration
	retry    uint

	request  *http.Request
	response *http.Response
	err      error
}

func New() *Request {
	return &Request{
		request: &http.Request{},
		headers: map[string]string{},
		params:  map[string]interface{}{},
	}
}

func (req *Request) SetURL(path string) *Request {
	req.url = path
	return req
}
func (req *Request) SetParams(params map[string]any) *Request {
	if req.params == nil {
		req.params = params
	} else {
		for k, v := range params {
			req.params[k] = v
		}
	}
	return req
}
func (req *Request) SetParam(k string, v interface{}) *Request {
	if req.params == nil {
		req.params = make(map[string]interface{})
	}
	req.params[k] = v
	return req
}
func (req *Request) SetBody(body interface{}) *Request {
	switch v := body.(type) {
	case string:
		req.body = v
	case []byte:
		req.body = string(v)
	default:
		req.bodyJSON = body
	}
	return req
}
func (req *Request) SetHeader(k, v string) *Request {
	if req.headers == nil {
		req.headers = make(map[string]string)
	}
	req.headers[k] = v
	return req
}
func (req *Request) SetTimeout(t time.Duration) *Request {
	req.timeout = t
	return req
}
func (req *Request) SetRetry(retry uint) *Request {
	req.retry = retry
	return req
}
func (req *Request) SetContentType(a string) *Request {
	req.SetHeader("Content-Type", a)
	return req
}
